/*
    I/O level commands implementation.
    Copyright (C) 2010 2011 2016 2020  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file    cmd_io.h
     \author  Alessandro Accattini
     \date    18 Gen 2020
     \brief   Jim-TCL I/O commands implementation.
  */

/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <string.h>
#include "drivers/drivers.h"			// direct I/O control

#include "cmd_io.h"				// my header file

/***************************************************************************/
/* costanti pubbliche */

/*
const char	baudrateShortHelp[]=	"Set the hardware baudrate";
const char	baudrateHelp[]=		"<baudrate>\n\n"
		"Set the hardware auto update baudrate";
const char	pindirShortHelp[]=	"Set single pin direction";
const char	pindirHelp[]=		"[pin_number] [in|out]\n\n"
		"Set the direction of given pin, without parametrs show the\n"
		"I/O pins direction";
const char	pinrShortHelp[]=	"Read input pin status";
const char	pinrHelp[]=		"[pin_number]\n\n"
		"Read the instantaneus input level on specified pin, if any "
		"pin is specified\nall pins are read";
const char	pinwShortHelp[]=	"Write output pin status";
const char	pinwHelp[]=		"<pin_number> <hi|lo>\n\n"
		"Set the new level for an output pin";
*/		

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int baudrate_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Set the hardware interface baudrate. */



  long int	BaudRate;		// nuovo baudrate



  if(argc!=2)				// se gli argomenti sono sbagliati
  {
    // errore: mancano dei parametri essenziali
    Jim_WrongNumArgs(Interp,1,argv,"baudrate");		// right usage
    return JIM_ERR;					// termina con errore
  }
  if(Jim_GetLong(Interp,argv[1],&BaudRate) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }
  IO_SetBaudRate(BaudRate);				// imposta il baudrate
  Jim_SetResult(Interp,argv[1]);			// command return value
  return JIM_OK;					// termina con successo
}



int dirmask_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Imposta/legge la direzione dei pin di I/O. */
  
  
  
  long int	Mask;			// new dirmask



  if(argc < 2)				// se manca il nuovo valore
  {
    // ritorna il valore attualmente impostato
    Jim_SetResultInt(Interp,IO_DirMaskRead());		// direzione attuale
    return JIM_OK;					// termina con successo
  }

  // imposta il registro direzione
  if(Jim_GetLong(Interp,argv[1],&Mask) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }
  IO_DirMaskWrite((BusData_t)Mask);			// modifica la dir.
  Jim_SetResult(Interp,argv[1]);			// command return value
  return JIM_OK;					// termina con successo
}



int dirbit_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Sets/gets a single bit in the I/O direction register. */
  
  
  
  long int	Bit;			// bit number
  long int	Dir;			// bit direction



  if(argc < 2)				// se manca il bit number
  {
    // errore: mancano dei parametri essenziali
    Jim_WrongNumArgs(Interp,1,argv,"bit_num ?bit_dir?");	// right usage
    return JIM_ERR;					// termina con errore
  }

  // ha almeno il primo parametro, quindi va convertito
  if(Jim_GetLong(Interp,argv[1],&Bit) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }
  if(argc < 3)				// se manca la direzione
  {
    // ritorna il valore attualmente impostato
    Jim_SetResultInt(Interp,IO_DirMaskBitRead(Bit));	// direzione attuale
    return JIM_OK;					// termina con successo
  }
    
  // converte il secondo parametro
  if(Jim_GetLong(Interp,argv[2],&Dir) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[2]);
    return JIM_ERR;					// termina con errore
  }

  // imposta il registro direzione
  IO_DirMaskBitWrite(Bit,Dir);				// modifica la dir.
  Jim_SetResult(Interp,argv[2]);			// command return value
  return JIM_OK;					// termina con successo
}



int dataout_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Sets/gets the I/O port output data register value. */
  
  
  
  long int	Data;			// new output data value



  if(argc < 2)				// se manca il nuovo valore
  {
    // ritorna il valore attualmente impostato
    Jim_SetResultInt(Interp,IO_OutDataRead());		// valore attuale
    return JIM_OK;					// termina con successo
  }

  // modifica l'output register
  if(Jim_GetLong(Interp,argv[1],&Data) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }
  IO_OutDataWrite((BusData_t)Data);			// output value
  Jim_SetResult(Interp,argv[1]);			// command return value
  return JIM_OK;					// termina con successo
}



int datain_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Gets the I/O port input data register value. */
  
  
  
  // ritorna il valore attualmente in ingresso
  Jim_SetResultInt(Interp,IO_InDataRead());		// ingresso attuale
  return JIM_OK;					// termina con successo
}



int bitset_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Sets a single bit in the I/O output data register. */
  
  
  
  long int	Bit;			// bit number
  long int	Value;			// bit value



  if(argc < 3)				// se mancano gli argomenti
  {
    // errore: mancano dei parametri essenziali
    Jim_WrongNumArgs(Interp,1,argv,"bit_num bit_value");	// right usage
    return JIM_ERR;					// termina con errore
  }

  // converte gli argomenti
  if(Jim_GetLong(Interp,argv[1],&Bit) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }
  if(Jim_GetLong(Interp,argv[2],&Value) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[2]);
    return JIM_ERR;					// termina con errore
  }

  // imposta il bit in uscita
  IO_BitWrite(Bit,Value);				// modifica il bit
  Jim_SetResult(Interp,argv[2]);			// command return value
  return JIM_OK;					// termina con successo
}



int bitget_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Gets a single bit value from the I/O input data register. */
  
  
  
  long int	Bit;			// bit number



  if(argc < 2)				// se mancano gli argomenti
  {
    // errore: mancano dei parametri essenziali
    Jim_WrongNumArgs(Interp,1,argv,"bit_num");		// right usage
    return JIM_ERR;					// termina con errore
  }

  // converte gli argomenti
  if(Jim_GetLong(Interp,argv[1],&Bit) != JIM_OK)
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }

  // legge il bit
  Jim_SetResultInt(Interp,IO_BitRead(Bit));		// ingresso attuale
  return JIM_OK;					// termina con successo
}

/* $Source$ */

