/*
    I/O level commands implementation.
    Copyright (C) 2010 2011 2016 2020  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

#ifndef __CMD_IO_H
#define __CMD_IO_H	1


#include "../jimtcl/jim.h"			// Jim Interpreter data types

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* macro pubbliche */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

int baudrate_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv);
  /**< \brief  Sets the hardware interface baudrate.
       \param Interp    Jim-TCL interpreter.
       \param argc      Number of arguments.
       \param argv      List of arguments.
       \return          JIM_OK on success.

       A single argument is expected: new baudrate.
   */

int dirmask_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv);
  /**< \brief  Sets/gets the I/O port direction register value.
       \param Interp    Jim-TCL interpreter.
       \param argc      Number of arguments.
       \param argv      List of arguments.
       \return          JIM_OK on success.

       A single argument is expected: new direction register value.
       Without arguments actual register value is returned.
   */

int dirbit_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv);
  /**< \brief  Sets/gets a single bit in the I/O direction register.
       \param Interp    Jim-TCL interpreter.
       \param argc      Number of arguments.
       \param argv      List of arguments.
       \return          JIM_OK on success.

       Two arguments are expected: bit number and bit direction.
       Bit number is always mandatory, if bit direction is not specified
       actual bit direction is returned.
   */

int dataout_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv);
  /**< \brief  Sets/gets the I/O port output data register value.
       \param Interp    Jim-TCL interpreter.
       \param argc      Number of arguments.
       \param argv      List of arguments.
       \return          JIM_OK on success.

       A single argument is expected: new output register value.
       Without arguments actual register value is returned.
   */

int datain_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv);
  /**< \brief  Gets the I/O port input data register value.
       \param Interp    Jim-TCL interpreter.
       \param argc      Number of arguments.
       \param argv      List of arguments.
       \return          JIM_OK on success.

       No arguments are expected.
   */

int bitset_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv);
  /**< \brief  Sets a single bit in the I/O output data register.
       \param Interp    Jim-TCL interpreter.
       \param argc      Number of arguments.
       \param argv      List of arguments.
       \return          JIM_OK on success.

       Two arguments are expected: bit number and bit value; both are
       mandatory.
       If an input configured bit is specified nothing appens.
   */

int bitget_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv);
  /**< \brief  Gets a single bit value from the I/O input data register.
       \param Interp    Jim-TCL interpreter.
       \param argc      Number of arguments.
       \param argv      List of arguments.
       \return          JIM_OK on success.

       A single arguments is expected: bit number, it is mandatory.
       If an output configured bit is specified its value is returned.
   */

/***************************************************************************/
#endif

/* $Source$ */

