/*
    SPI bus commands implementation.
    Copyright (C) 2010 2011 2016 2020  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file    cmd_spi.c
     \author  Alessandro Accattini
     \date    18 Gen 2020
     \brief   Jim-TCL I/O commands implementation.
  */

/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <string.h>
#include "drivers/drivers.h"			// direct I/O control

#include "cmd_spi.h"				// my header file

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int spi_init_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Initializes SPI bus. */



  long int	Mode;			// SPI bus mode
  long int	WordLen;		// bus word length



  if(argc < 3)				// se gli argomenti sono sbagliati
  {
    // errore: mancano dei parametri essenziali
    Jim_WrongNumArgs(Interp,1,argv,"mode word_len");	// right usage
    return JIM_ERR;					// termina con errore
  }
  if(Jim_GetLong(Interp,argv[1],&Mode) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }
  if(Jim_GetLong(Interp,argv[2],&WordLen) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[2]);
    return JIM_ERR;					// termina con errore
  }
  Jim_SetResultInt(Interp,SPI_InitBus(Mode,WordLen));	// configura il bus
  return JIM_OK;					// termina con successo
}



int spi_xchg_Cmd(Jim_Interp *Interp,int argc,Jim_Obj *const *argv)
{
  /* Send/receive a word over SPI bus. */

  
  
  long int	DataOut;		// sent data



  if(argc < 2)				// se gli argomenti sono sbagliati
  {
    // errore: mancano dei parametri essenziali
    Jim_WrongNumArgs(Interp,1,argv,"out_data");		// right usage
    return JIM_ERR;					// termina con errore
  }
  if(Jim_GetLong(Interp,argv[1],&DataOut) != JIM_OK)	// se non converte
  {
    Jim_SetResultFormatted(Interp,"invalid value \"%#s\"",argv[1]);
    return JIM_ERR;					// termina con errore
  }
  
  // ritorna il valore ricevuto in ingresso
  Jim_SetResultInt(Interp,SPI_DataExchange(DataOut));	// SPI send/recv
  return JIM_OK;					// termina con successo
}

/* $Source$ */

