/*
    Commands library header file.
    Copyright (C) 2010 2011 2016 2020  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file    commands.c
     \author  Alessandro Accattini
     \date    11 Apr 2020
     \brief   Jim-TCL Construct commands implementation.
  */

/***************************************************************************/
/* include vari */

#include "drivers/io.h"				// base I/O access
#include "cmd_io.h"				// I/O control commands
#include "cmd_spi.h"				// SPI bus control commands

#include "commands.h"				// my header file

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* macro interne alla libreria */

#define NEW_COMMAND(cmd,func)	if((Err=Jim_CreateCommand(Interp,cmd,func,NULL,NULL))) return(Err);

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int Jim_AddConstructCommands(Jim_Interp *Interp)
{
  /* Adds all specific Construct commands to interpreter. */



  int	Err;



  // generic I/O control commands
  NEW_COMMAND("port_baudrate",	baudrate_Cmd);
  NEW_COMMAND("port_dir_mask",	dirmask_Cmd);
  NEW_COMMAND("port_dir_bit",	dirbit_Cmd);
  NEW_COMMAND("port_dout",	dataout_Cmd);
  NEW_COMMAND("port_din",	datain_Cmd);
  NEW_COMMAND("port_bit_set",	bitset_Cmd);
  NEW_COMMAND("port_bit_get",	bitget_Cmd);
  
  // SPI commands
  NEW_COMMAND("spi_init",	spi_init_Cmd);
  NEW_COMMAND("spi_xchg",	spi_xchg_Cmd);
  return(JIM_OK);				// termina con successo
}

/* $Source$ */

