/*
    AT24C02A family driver implementation.
    Copyright (C) 2011 2012 2016 2018  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __AT24C02A_C
#include "at24c02a.h"				// functions prototypes

#include "i2c.h"				// I2C bus control

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

static const char	LinkMap[]=	"D0 <---> SDA\n"
					"D1 ----> SCL\n"
					"D2  ---  nc\n"
					"D3  ---  nc\n"
					"D4  ---  nc\n"
					"D5  ---  nc\n"
					"D6  ---  nc\n"
					"D7  ---  nc\n";

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

static int AT24C02A_InitBus(void *Chipset,int argc,char **argv);
  /* Imposta la direzione e lo stato iniziale delle linee del bus. */

static int AT24C02A_TextWrite(Addr_t Address,const char *Text,int Len);
  /* Scrive una stringa di testo all'indirizzo indicato. */

static int AT24C02A_TextRead(Addr_t Address,char *Text,int Len);
  /* Legge una stringa di Len char dall'indirizzo indicato. */

static int AT24C02A_BlockWrite(Addr_t Address,const Byte_t *Data,int Len);
  /* Scrive un blocco dati all'indirizzo indicato. */

static int AT24C02A_BlockRead(Addr_t Address,Byte_t *Buf,int Len);
  /* Legge un blocco dati dall'indirizzo indicato. */

/***************************************************************************/
/* costanti pubbliche */

CHIP_SET(	AT24C02A,			// struct name
                "AT24C02A",			// driver name
                LinkMap,			// hardware link map

                AT24C02A_InitBus,		// bus init method
                NULL,				// command write method
                AT24C02A_TextWrite,		// data write method
                AT24C02A_TextRead,		// data read method
                AT24C02A_BlockWrite,		// block write method
                AT24C02A_BlockRead,		// block read method

                8,				// block size
                32);				// num of blocks
CHIP_SET(	AT24C04A,			// struct name
                "AT24C04A",			// driver name
                LinkMap,			// hardware link map

                AT24C02A_InitBus,		// bus init method
                NULL,				// command write method
                AT24C02A_TextWrite,		// data write method
                AT24C02A_TextRead,		// data read method
                AT24C02A_BlockWrite,		// block write method
                AT24C02A_BlockRead,		// block read method

                16,				// block size
                32);				// num of blocks
CHIP_SET(	AT24C08A,			// struct name
                "AT24C08A",			// driver name
                LinkMap,			// hardware link map

                AT24C02A_InitBus,		// bus init method
                NULL,				// command write method
                AT24C02A_TextWrite,		// data write method
                AT24C02A_TextRead,		// data read method
                AT24C02A_BlockWrite,		// block write method
                AT24C02A_BlockRead,		// block read method

                16,				// block size
                64);				// num of blocks
CHIP_SET(	AT24C16A,			// struct name
                "AT24C16A",			// driver name
                LinkMap,			// hardware link map

                AT24C02A_InitBus,		// bus init method
                NULL,				// command write method
                AT24C02A_TextWrite,		// data write method
                AT24C02A_TextRead,		// data read method
                AT24C02A_BlockWrite,		// block write method
                AT24C02A_BlockRead,		// block read method

                16,				// block size
                128);				// num of blocks

/***************************************************************************/
/* funzioni */

static int AT24C02A_InitBus(void *Chipset,int argc,char **argv)
{
  /* Imposta la direzione e lo stato iniziale delle linee del bus. */



  DrvInt_t	*Driver=(DrvInt_t*)Chipset;	// punta alla struct del driver



  return(I2C_InitBus(Driver->BlockSize,8+3,0xa0));
}



static int AT24C02A_TextWrite(Addr_t Address,const char *Text,int Len)
{
  /* Scrive una stringa di testo all'indirizzo indicato. */



  while(Len--)
  {
    if(I2C_WriteByte(Address++,(unsigned char)*Text++))
      return 1;					// termina con fallimento
  }
  return 0;					// termina con successo
}



static int AT24C02A_TextRead(Addr_t Address,char *Text,int Len)
{
  /* Legge una stringa di Len char dall'indirizzo indicato. */



  while(Len--)
  {
    if(I2C_ReadByte(Address++,(unsigned char*)Text++))
      return 1;					// termina con fallimento
  }
  return 0;					// termina con successo
}



static int AT24C02A_BlockWrite(Addr_t Address,const Byte_t *Data,int Len)
{
  /* Scrive un blocco dati all'indirizzo indicato. */



  return(I2C_WritePage(Address,Data));
}



static int AT24C02A_BlockRead(Addr_t Address,Byte_t *Buf,int Len)
{
  /* Legge un blocco dati dall'indirizzo indicato. */
  
  
  
  return(I2C_ReadPage(Address,Buf));
}

/* $Source$ */

