/*
    Supported chipset list.
    Copyright (C) 2011 2012 2014 2015 2016 2018  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __CHIPSET_C
#include "chipset.h"				// functions prototypes

#include <stdio.h>				// NULL
#include <string.h>				// strcmp

#include "hd44780.h"
#include "at24c02a.h"
#include "ssd1328.h"
#include "pcf8833.h"
#include "s1d15g10.h"
#include "pscard.h"

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* macro interne alla libreria */

#define END_OF_LIST()		NULL
#define IsEndOfList(pt)		(pt->BusInit == NULL)

#define FieldCheck(F)		Supp="    "; if(F == NULL) Supp="not "

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

static const DrvInt_t		*DriverList[]={
					&CHIP_SET_NAME(HD44780),
					&CHIP_SET_NAME(AT24C02A),
					&CHIP_SET_NAME(AT24C04A),
					&CHIP_SET_NAME(AT24C08A),
					&CHIP_SET_NAME(AT24C16A),

/*
  CHIP_SET(	"SSD1328",
  		"ssd1328.c",
		SSD1328_LinkMap,
		SSD1328_InitBus,
		SSD1328_CmdWrite,
		SSD1328_TextWrite,
		NULL,
		SSD1328_CopyFrame,
		NULL,
		64,			// block size
		128),			// num of blocks
  CHIP_SET(	"PCF8833",
  		"pcf8833.c",
		PCF8833_LinkMap,
		PCF8833_InitBus,
		PCF8833_CmdWrite,
		PCF8833_TextWrite,
		NULL,
		PCF8833_CopyFrame,
		NULL,
		64,			// block size
		128),			// num of blocks
  CHIP_SET(	"S1D15G10",
  		"s1d15g10.c",
		S1D15G10_LinkMap,
		S1D15G10_InitBus,
		S1D15G10_CmdWrite,
		S1D15G10_TextWrite,
		NULL,
		S1D15G10_CopyFrame,
		NULL,
		64,			// block size
		128),			// num of blocks
  CHIP_SET(	"PSCARD",
  		"pscard.c",
		PSCARD_LinkMap,
		PSCARD_InitBus,
		NULL,
		NULL,
		NULL,
		PSCARD_BlockWrite,
		PSCARD_BlockRead,
		128,			// block size
		32),			// num of blocks
*/


					END_OF_LIST()};

/***************************************************************************/
/* variabili interne alla libreria */

const DrvInt_t	**ListEntry=DriverList;		// puntatore alla lista

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

const DrvInt_t *DriverFirst(void)
{
  /**< \brief  Returns the first driver in the list.
       \return The first driver in the list.

       This function, together with DriverNext(), constitutes the way to
       access the driver list. A call to DriverFirst() always returns the
       first list entry, but also resets a list pointer used by DriverNext()
       to browse the list.
   */



  ListEntry=DriverList;			// punta a inizio lista
  return(*ListEntry);			// termina
}



const DrvInt_t *DriverNext(void)
{
  /**< \brief  Returns the next driver in the list.
       \return The next driver in the list or NULL if list ended.

       The driver list scan is alway started calling DriverFirst().
       DriverNext() returns the next list entry if it exist or NULL if the
       end of list is encountered.
   */



  if(*ListEntry == NULL)		// se ha raggiunto la fine della lista
    return(NULL);			// raggiunta fine lista
  ListEntry++;				// avanti Savoia!!
  return(*ListEntry);			// termina
}



const DrvInt_t *DriverSearch(const char *DrvName)
{
  /* Cerca un driver con il nome indicato e ritorna un puntatore al driver
     stesso, ritorna NULL se non esiste un driver col nome indicato.
     Chiamata con DrvName=NULL ritorna il primo chipset della lista. */
  
  
  
  const DrvInt_t	*DrvPtr;		// puntatore al driver



  DrvPtr=DriverFirst();				// punta al primo driver
  do
  {
    if(strcmp(DrvPtr->DrvName,DrvName) == 0)	// se trova il driver cercato
      return(DrvPtr);				// termina con successo
    DrvPtr=DriverNext();			// punta al driver successivo
  }while(DrvPtr != NULL);			// fino a fine lista
  return(NULL);					// termina senza aver trovato
}



void DriverPrintInfo(const DrvInt_t *DrvPtr,int Mode)
{
  /**< \brief  Print driver informations to stdout.
       \param  DrvPtr  A driver pointer.
       \param  Mode  Can be one value between: SHORT_INFO_MODE, LONG_INFO_MODE,
       		     FULL_INFO_MODE.
   */



  const char	*Supp;



  if(DrvPtr == NULL)				// no Martini?
    return;					// no party!

  // stampa il nome del driver
  if(Mode)					// se non e' SHORT
    putchar('\n');				// salta una riga
  printf("%s\n",DrvPtr->DrvName);

  // stampa tutte le info sul driver
  if(Mode & 0x01)
  {
    printf("  file              : %s\n",DrvPtr->DrvFile);
    printf("  block size        : %i\n",DrvPtr->BlockSize);
    printf("  blocks num        : %i\n",DrvPtr->BlocksNum);
    printf("  total size        : %i\n",
    		DrvPtr->BlockSize * DrvPtr->BlocksNum);
    FieldCheck(DrvPtr->CmdSend);
    printf("  command method    : %ssupported\n",Supp);
    FieldCheck(DrvPtr->CharWrite);
    printf("  char write method : %ssupported\n",Supp);
    FieldCheck(DrvPtr->CharRead);
    printf("  char read method  : %ssupported\n",Supp);
    FieldCheck(DrvPtr->BlockWrite);
    printf("  block write method: %ssupported\n",Supp);
    FieldCheck(DrvPtr->BlockRead);
    printf("  block read method : %ssupported\n",Supp);
  }

  // stampa il collegamento suggerito per il driver
  if(Mode & 0x02)
    printf("  link:\n%s\n",DrvPtr->LinkMap);
  return;					// termina
}



int DriverMaxBlock(void)
{
  /* Ritorna la dimensione del singolo blocco piu' grande tra tutti i
     chipset supportati. */
  
  
  
  const DrvInt_t	*DrvPtr;		// puntatore al driver
  int	Max=0;


  
  DrvPtr=DriverFirst();				// punta al primo driver
  do
  {
    if(DrvPtr->BlockSize > Max)			// se piu' del massimo
      Max=DrvPtr->BlockSize;			// aggiorna il massimo
    DrvPtr=DriverNext();			// punta al driver successivo
  }while(DrvPtr != NULL);			// fino a fine lista
  return(Max);					// termina
}



int DriverMaxSize(void)
{
  /* Ritorna la dimensione totale piu' grande tra tutti i chipset supportati. */
  
  
  
  const DrvInt_t	*DrvPtr;		// puntatore al driver
  int	Max=0;
  int	Total;
  
  
  
  DrvPtr=DriverFirst();				// punta al primo driver
  do
  {
    Total=DrvPtr->BlockSize * DrvPtr->BlocksNum;
    if(Total > Max)				// se piu' del massimo
      Max=Total;				// aggiorna il massimo
    DrvPtr=DriverNext();			// punta al driver successivo
  }while(DrvPtr != NULL);			// fino a fine lista
  return(Max);					// termina
}

/* $Source$ */

