/*
    Supported driver list.
    Copyright (C) 2011 2014 2015 2018  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file
     \author  Alessandro Accattini
     \date    9 Gen 2018
     \brief   Driver list handling functions.

     The full driver list and all list handling functions are implemented in
     this file.
 */


#ifndef __CHIPSET_H
#define __CHIPSET_H	1


#include "drv_type.h"				// driver data type

/***************************************************************************/
/* define pubblici */

  // valore per il campo Mode di DriverPrintInfo()
#define SHORT_INFO_MODE			0x00
#define LONG_INFO_MODE			0x01
#define FULL_INFO_MODE			0x03

/***************************************************************************/
/* macro pubbliche */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

const DrvInt_t *DriverFirst(void);
  /**< \brief  Returns the first driver in the list.
       \return The first driver in the list.

       This function, together with DriverNext(), constitutes the way to
       access the driver list. A call to DriverFirst() always returns the
       first list entry, but also resets a list pointer used by DriverNext()
       to browse the list.
   */

const DrvInt_t *DriverNext(void);
  /**< \brief  Returns the next driver in the list.
       \return The next driver in the list or NULL if list ended.

       The driver list scan is alway started calling DriverFirst().
       DriverNext() returns the next list entry if it exist or NULL if the
       end of list is encountered.
   */

const DrvInt_t *DriverSearch(const char *DrvName);
  /**< \brief  Serach for a specific driver.
       \param  DrvName  The driver name to search for.
       \return A driver pointer or NULL if the driver is unknown.

       This function let you search for a specific driver without browsing
       the full driver list.

       WARNING: this functions makes use of DriverFirst() and DriverNext() to
       browse the driver list, so it is not a good idea calling DriverSearch()
       in the middle of a driver list scan.
   */

void DriverPrintInfo(const DrvInt_t *DrvPtr,int Mode);
  /**< \brief  Print driver informations to stdout.
       \param  DrvPtr  A driver pointer.
       \param  Mode  Can be one value between: SHORT_INFO_MODE, LONG_INFO_MODE,
       		     FULL_INFO_MODE.
   */


int DriverMaxBlock(void);
  /**< \brief  Give the size of larger data block in the driver list.
       \return The size of the bigger data block.
   */

int DriverMaxSize(void);
  /**< \brief  Give the size of larger total data block in the driver list.
       \return The size of the bigger total data block.
   */

/***************************************************************************/
#endif

/* $Source$ */

