/*
    Function trace helper.
    Copyright (C) 2018  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file    trace.h
     \author  Alessandro Accattini
     \date    4 Gen 2018
     \brief   Function trace macros declaration.

     This file defines a set of macros used to trace drivers functions calls
     and to display messages from drivers itself.
     It is possible to disable both to increase processing speed.
 */


#ifndef __TRACE_H
#define __TRACE_H	1


#include <stdio.h>				// printf()

/***************************************************************************/
/* define pubblici */

#ifndef ENABLE_FUNCTION_MESSAGES
#define ENABLE_FUNCTION_MESSAGES	1	// enabled if !0
#endif

#ifndef ENABLE_FUNCTION_TRACE
#define ENABLE_FUNCTION_TRACE		0	// enabled if !0
#endif

  // vebosity levels
#define DRIVER_VERB_LEVEL		3	// livello driver
#define BUS_VERB_LEVEL			4	// livello bus
#define IO_VERB_LEVEL			5	// livello I/O

/***************************************************************************/
/* macro pubbliche */

#if ENABLE_FUNCTION_MESSAGES
#define MESSAGE(verb_level,...)	if(Verbosity >= verb_level) printf(__VA_ARGS__)
#else
#define MESSAGE(verb_level,...)
#endif


#if ENABLE_FUNCTION_TRACE
#define FUNCTION_ENTER()	printf("%s(): entering\n",__FUNCTION__)
#define FUNCTION_LEAVE()	printf("%s(): leaving\n",__FUNCTION__)
#else
#define FUNCTION_ENTER()
#define FUNCTION_LEAVE()
#endif

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* variabili pubbliche */

#ifdef __TRACE_C

#else

extern int		Verbosity;		// verbosity level

#endif

/***************************************************************************/
/* funzioni pubbliche */

void SetVebosityLevel(int NewLevel);
  /**< \brief  Set verbosity level.
       \param NewLevel  new verbosity level.

       This function set the vebosity level of all drivers functions.
       Higher verbosity level is usually used during driver debugging.
   */

/***************************************************************************/
#endif

/* $Source$ */

