/*
    Char device accessing functions.
    Copyright (C) 2008  Alessandro Accattini


 *****************************************************************************
 *
 *
 *
 */



/***************************************************************************/
/* include vari */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/uaccess.h>			// user space access

#include "debug.h"				// debug and print macros
#include "oledf.h"				// display control
//#include "io.h"					// hardware control

/***************************************************************************/
/* define interni alla libreria */

#define CHARBUFFER_SIZE		1024		// dimensione char buffer

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static unsigned char	CharBuffer[CHARBUFFER_SIZE];	// char buffer

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

ssize_t CharRead(struct file *Filp,char __user *Buff,
			size_t Count,loff_t *Offp);
  /* Read method, read data from the device. */

ssize_t CharWrite(struct file *Filp,const char __user *Buff,
			size_t Count,loff_t *Offp);
  /* Write method, write data to the device. */

loff_t CharSeek(struct file *Filp,loff_t Off,int Whence);
  /* Seek method, riposiziona il puntatore R/W al file. */

/***************************************************************************/
/* funzioni */

ssize_t CharRead(struct file *Filp,char __user *Buff,
			size_t Count,loff_t *Offp)
{
  /* Read method, read data from the device. */



  /* debug info */
  PDEBUG("%s(Count=%li,Off=%li)\n",__FUNCTION__,
		(unsigned long int)Count,(unsigned long int)*Offp);
  
  PDEBUG("%s: read not allowed, return 0\n",__FUNCTION__);
  return 0;				// termina senza lettura
}



ssize_t CharWrite(struct file *Filp,const char __user *Buff,
			size_t Count,loff_t *Offp)
{
  /* Write method, write data to the device. */
  
  
  
  unsigned long int	MaxWriteBytes;



  /* debug info */
  PDEBUG("%s(Count=%li,Off=%li)\n",__FUNCTION__,
		(unsigned long int)Count,(unsigned long int)*Offp);
  
  /* determina il numero massimo di bytes scrivibili sul device */
  if(*Offp>=CHARBUFFER_SIZE)
  {
    PDEBUG("%s: no write at all, return 0\n",__FUNCTION__);
    return 0;				// termina senza lettura
  }
  MaxWriteBytes=CHARBUFFER_SIZE-(unsigned long int)*Offp;
  if(Count>MaxWriteBytes)
    Count=MaxWriteBytes;		// limita la scrittura

  /* effettua la scrittura sul device */
  PDEBUG("%s: copying data from user space\n",__FUNCTION__);
  while(down_interruptible(&HardSem));	// blocca il semaforo
  if(copy_from_user(&CharBuffer[*Offp],Buff,(unsigned long int)Count))
  {
    up(&HardSem);			// libera il semaforo
    return(-EFAULT);			// segmentation fault
  }
  PDEBUG("%s: refreshing display image\n",__FUNCTION__);
  OledTurnOnDisplay();			// abilita il display
  OledPrintStr(CharBuffer,Count);	// stampa la stringa sul display
//  *Offp+=Count;				// incrementa il puntatore
  up(&HardSem);				// libera il semaforo
  PDEBUG("%s: display data updated\n",__FUNCTION__);
  return((ssize_t)Count);		// termina con successo
}



loff_t CharSeek(struct file *Filp,loff_t Off,int Whence)
{
  /* Seek method, riposiziona il puntatore R/W al file. */



  Filp->f_pos=0;			// blocca lo scorrimento del file
  return(-EINVAL);			// seek invalid, termina con errore
}

/* $Source: /users/CVS/oled_linux/chardev.c,v $ */

