/*
    LED handling functions.
    Copyright (C) 2008 2009  Alessandro Accattini


 *****************************************************************************
 *
 *
 *
 */



/***************************************************************************/
/* include vari */

#include <linux/module.h>
#include <linux/delay.h>			// ndelay, udelay, mdelay
#include <linux/timer.h>			// kernel software timers

#include "debug.h"				// debug and print macros
#include "io.h"					// I/O level access

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

//static struct timer_list	RedTimer;	// timer led rosso
//static struct timer_list	OrangeTimer;	// timer led arancione
static struct timer_list	GreenTimer;	// timer led verde

/***************************************************************************/
/* funzioni interne alla libreria */

static void GreenLedTimeElapsed(unsigned long Data);
  /* Spegne il led verde allo scadere del tempo programmato. */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

void LedInit(void);
  /* Inizializza i led e la loro gestione. */

void LedDeinit(void);
  /* Elimina le risorse allocate alla gestione dei led. */

void LedGreen(int On);
  /* Accende il led verde e riarma il timer automatico o spegne il led
     e distrugge il timer. */

/***************************************************************************/
/* funzioni */

static void GreenLedTimeElapsed(unsigned long Data)
{
  /* Spegne il led verde allo scadere del tempo programmato. */



  PDEBUG("%s: GREEN led auto-shut-off\n",__FUNCTION__);
  Set_LED3(0);					// spegne il led VERDE
  return;					// termina
}



void LedInit(void)
{
  /* Inizializza i led e la loro gestione. */



  PDEBUG("%s()\n",__FUNCTION__);		// debug call info
  
  /* inizializza i timer per lo spegnimento */
  init_timer(&GreenTimer);			// inizializza la struttura
  GreenTimer.function=GreenLedTimeElapsed;	// imposta la funzione chiamata
  GreenTimer.expires=jiffies+(20*HZ);		// imposta la scadenza
  add_timer(&GreenTimer);			// avvia il timer
  Set_LED3(1);					// accende il led VERDE
  return;					// termina
}



void LedDeinit(void)
{
  /* Elimina le risorse allocate alla gestione dei led. */
  
  
  
  del_timer_sync(&GreenTimer);			// distrugge il timer
  return;					// termina
}



void LedGreen(int On)
{
  /* Accende il led verde e riarma il timer automatico o spegne il led
     e distrugge il timer. */



  if(On)
  {
    PDEBUG("%s: GREEN led turn-on\n",__FUNCTION__);
    Set_LED3(1);				// accende il led VERDE
    mod_timer(&GreenTimer,jiffies+(5*HZ));	// riarma il timer
  }
  else
  {
    PDEBUG("%s: GREEN led turn-off\n",__FUNCTION__);
    if(timer_pending(&GreenTimer))		// se il timer e' schedulato
      del_timer_sync(&GreenTimer);		// distrugge il timer
    Set_LED3(0);				// spegne il led VERDE
  }
  return;					// termina
}

/* $Source: /users/CVS/oled_linux/led.c,v $ */

