#
# HD44780 control functions implementation.
#


#
# I/O bonding and bus direction control
#
proc HD44780_Set_E  {Value} {port_bit_set 2 $Value}
proc HD44780_Set_Rw {Value} {port_bit_set 1 $Value}
proc HD44780_Set_Dc {Value} {port_bit_set 0 $Value}
proc HD44780_Bus_Out {} {port_dir_mask 0xff}
proc HD44780_Bus_In  {} {port_dir_mask 0x0f}



#
# Generic HD44780 half write operation
#
proc HD44780_Half_Write {Data} {
  HD44780_Set_Rw 0					;# bus write operation
  HD44780_Bus_Out					;# bus direction out
  set CtrlBus [expr [port_din] & 0x0f]			;# control bus status
  port_dout [expr ($Data & 0xf0) | $CtrlBus]		;# high nibble on bus
  HD44780_Set_E 1					;# E=1
  HD44780_Set_E 0					;# E=0
  HD44780_Bus_In					;# bus direction in
  HD44780_Set_Rw 1					;# bus read operation
}



#
# Generic HD44780 write operation
#
proc HD44780_Write {Data} {
  HD44780_Set_Rw 0					;# bus write operation
  HD44780_Bus_Out					;# bus direction out
  set CtrlBus [expr [port_din] & 0x0f]			;# control bus status
  port_dout [expr ($Data & 0xf0) | $CtrlBus]		;# high nibble on bus
  HD44780_Set_E 1					;# E=1
  HD44780_Set_E 0					;# E=0
  port_dout [expr (($Data << 4) & 0xf0) | $CtrlBus]	;# low nibble on bus
  HD44780_Set_E 1					;# E=1
  HD44780_Set_E 0					;# E=0
  HD44780_Bus_In					;# bus direction in
  HD44780_Set_Rw 1					;# bus read operation
}



#
# Single byte write data operation
#
proc HD44780_Write_Data {Data} {
  while {[HD44780_Get_Busy]} {}				;# whait for ready flag
  HD44780_Set_Dc 1					;# data mode
  HD44780_Write $Data					;# data write
}



#
# Single byte write command operation
#
proc HD44780_Write_Command {Cmd} {
  while {[HD44780_Get_Busy]} {}				;# whait for ready flag
  HD44780_Set_Dc 0					;# command mode
  HD44780_Write $Cmd					;# command write
}



#
# Generic HD44780 read operation
#
proc HD44780_Read {} {
  HD44780_Set_E 1					;# E=1
  set High [expr [port_din] & 0xf0]			;# high nibble read
  HD44780_Set_E 0					;# E=0
  HD44780_Set_E 1					;# E=1
  set Low [expr [port_din] & 0xf0]			;# low nibble read
  HD44780_Set_E 0					;# E=0
  return [expr $High | ($Low >> 4)]			;# read 8 bit value
}



#
# Single byte read data operation
#
proc HD44780_Read_Data {} {
  HD44780_Set_Dc 1					;# data mode
  return [HD44780_Read]					;# data read
}



#
# Single byte write command operation
#
proc HD44780_Read_Status {} {
  HD44780_Set_Dc 0					;# command mode
  return [HD44780_Read]					;# status read
}



#
# Returns the HD44780 busy flag value.
#
proc HD44780_Get_Busy {} {
  return [expr ([HD44780_Read_Status] & 0x80) || 0]	;# busy flag only
}



#
# Prints text string.
#
proc HD44780_Print {Text} {
  set Len [string length $Text]				;# string length
  for {set i 0} {$i<$Len} {incr i} {
    HD44780_Write_Data [scan [string index $Text $i] %c]	;# single char
  }
}



#
# HD44780 initialization procedure
#
proc HD44780_Init {} {
  HD44780_Bus_In			;# bus direction in
  HD44780_Set_E 0			;# E=0
  HD44780_Set_Rw 1			;# read mode
  HD44780_Set_Dc 0			;# command mode
  HD44780_Half_Write 0x30		;# ensures 8 bit mode
  HD44780_Half_Write 0x30
  HD44780_Half_Write 0x30
  HD44780_Half_Write 0x20		;# 4 bit bus mode
  HD44780_Write_Command 0x28		;# 4 bit, 2 lines, font 5x8
  HD44780_Write_Command 0x0f		;# display ON, cursor ON, blinking ON
  HD44780_Write_Command 0x01		;# clear display
}


