#
# Oled display control functions implementation.
#


#
# I/O bonding
#
proc SSD1328_Set_CS  {Value} {port_bit_set 3 $Value}
proc SSD1328_Set_DC  {Value} {port_bit_set 4 $Value}
proc SSD1328_Set_RES {Value} {port_bit_set 5 $Value}
proc SSD1328_Set_NEN {Value} {port_bit_set 6 $Value}
proc SSD1328_Set_SEL {Value} {port_bit_set 7 $Value}


#
# Generic SSD1328 command write operation
#   Data is a list of bytes
#
proc SSD1328_Command {Data} {
  SSD1328_Set_DC 0				;# command mode
  SSD1328_Set_CS 0				;# abbassa CS
  foreach Value $Data {
    spi_xchg $Value				;# invia un byte al controller
  }
  SSD1328_Set_CS 1				;# alza CS
}


#
# Generic SSD1328 data write operation
#   Data is a binary string
#
proc SSD1328_Data {Data} {
  set Len [string length $Data]			;# string length
  SSD1328_Set_DC 1				;# data mode
  SSD1328_Set_CS 0				;# abbassa CS
  for {set i 0} {$i<$Len} {incr i} {
    spi_xchg [scan [string index $Data $i] %c]	;# single string byte
  }
  SSD1328_Set_DC 0				;# command mode
  spi_xchg 0xe3					;# aggiunge un NOP
  SSD1328_Set_CS 1				;# alza CS
}


#
# Prints full binary image data
#
proc SSD1328_FullImage {Data} {
  set SSD1328_ON {0xaf}
  set SSD1328_GOTO_X {0x15 0 63}
  set SSD1328_GOTO_Y {0x75 0 127}
  
  ;# riscrive tutto il display
  SSD1328_Command $SSD1328_ON
  SSD1328_Command $SSD1328_GOTO_X
  SSD1328_Command $SSD1328_GOTO_Y

  SSD1328_Data $Data				;# image binary data
}


#
# Prints text string.
#
#proc SSD1328_Print {Text} {
#  set Len [string length $Text]				;# string length
#  for {set i 0} {$i<$Len} {incr i} {
#    SSD1328_Write_Data [scan [string index $Text $i] %c]	;# single char
#  }
#}



#
# SSD1328 initialization procedure
#
proc SSD1328_Init {} {
  set SSD1328_INIT {0xad 0x02}
  set SSD1328_ON {0xaf}
  set SSD1328_OFF {0xae}
  set SSD1328_REMAP {0xa0 0x52}




  ;# inizializza il bus
  port_dir_bit 3 1				;# linee di controllo in uscita
  port_dir_bit 4 1
  port_dir_bit 5 1
  port_dir_bit 6 1
  port_dir_bit 7 1
  SSD1328_Set_CS 1				;# alza CS 
  SSD1328_Set_DC 0 
  SSD1328_Set_RES 0
  SSD1328_Set_NEN 0
  SSD1328_Set_SEL 0 
  spi_init 7 8					;# SPI bus init

  ;# resetta il display controller
  SSD1328_Set_CS 1				;# alza CS
  SSD1328_Set_RES 0				;# abbassa RESET
  after 120					;# aspetta 120ms
  SSD1328_Set_RES 1				;# alza RESET
  after 120

  ;# inizializza il display controller
  SSD1328_Command $SSD1328_INIT
  SSD1328_Command $SSD1328_ON
  SSD1328_Command $SSD1328_REMAP
}


